CREATE DATABASE ocsweb;
USE ocsweb;


-- Name
-- Caption (=Name?)
-- Description
-- Manufacturer
-- Version
-- Type


CREATE TABLE hardware (
	ID		INTEGER NOT NULL auto_increment,
	DEVICEID	VARCHAR(255) not NULL,
	NAME		VARCHAR(255) default NULL,
	WORKGROUP	VARCHAR(255) default NULL,
	USERDOMAIN	VARCHAR(255) default NULL,
	OSNAME		VARCHAR(255) default NULL,
	OSVERSION	VARCHAR(255) default NULL,
	OSCOMMENTS	VARCHAR(255) default NULL,
	PROCESSORT	VARCHAR(255) default NULL,
	PROCESSORS	INTEGER default 0,
	PROCESSORN	SMALLINT default NULL,
	MEMORY		INTEGER default NULL,
	SWAP		INTEGER default NULL,
	IPADDR		VARCHAR(255) default NULL,
	ETIME		DATETIME default NULL,
	LASTDATE	DATETIME default NULL,
	LASTCOME	DATETIME default NULL,
	QUALITY		DECIMAL(4,3) default 0,
	FIDELITY	BIGINT default 1,
	USERID		VARCHAR(255) default NULL,
	`TYPE`		INTEGER default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	WINCOMPANY	VARCHAR(255) default NULL,
	WINOWNER	VARCHAR(255) default NULL,
	WINPRODID	VARCHAR(255) default NULL,
	WINPRODKEY	VARCHAR(255) default NULL,
	USERAGENT	VARCHAR(50) default NULL,
	CHECKSUM	INTEGER default 0,
		PRIMARY KEY  (DEVICEID, ID),
		INDEX ID (ID),
		INDEX NAME (NAME),
		INDEX CHECKSUM (CHECKSUM),
		INDEX DEVICEID (DEVICEID)
) ENGINE=INNODB ;




CREATE TABLE accesslog (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	USERID		VARCHAR(255) default NULL,
	LOGDATE		DATETIME default NULL,
	PROCESSES	TEXT,
		INDEX ID (ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;



CREATE TABLE accountinfo (
	HARDWARE_ID	INTEGER NOT NULL,
	TAG		VARCHAR(255) default 'NA',
		primary key(HARDWARE_ID),
		INDEX TAG (TAG)
) ENGINE=INNODB ;



CREATE TABLE deploy (
	NAME		VARCHAR(255) NOT NULL,
	CONTENT 	LONGBLOB NOT NULL,
		PRIMARY KEY  (NAME)
) ENGINE=MYISAM ;


CREATE TABLE netmap (
	IP		VARCHAR(15) NOT NULL,
	MAC		VARCHAR(17) NOT NULL,
	MASK		VARCHAR(15) NOT NULL,
	NETID		VARCHAR(15) NOT NULL,
	DATE 		TIMESTAMP default CURRENT_TIMESTAMP,
	NAME		VARCHAR(255) default NULL,
		PRIMARY KEY  (MAC),
		INDEX IP (IP),
		INDEX NETID (NETID)
) ENGINE=INNODB ;



CREATE TABLE bios (
	HARDWARE_ID	INTEGER NOT NULL,
	SMANUFACTURER	VARCHAR(255) default NULL,
	SMODEL		VARCHAR(255) default NULL,
	SSN		VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
	BMANUFACTURER	VARCHAR(255) default NULL,
	BVERSION	VARCHAR(255) default NULL,
	BDATE		VARCHAR(255) default NULL,
		PRIMARY KEY  (HARDWARE_ID),
		INDEX SSN (SSN)
) ENGINE=INNODB ;


CREATE TABLE config (
	NAME		VARCHAR(50) NOT NULL,
	IVALUE		INTEGER default NULL,
	TVALUE		VARCHAR(255) default NULL,
	COMMENTS	TEXT,
		PRIMARY KEY (NAME)
) ENGINE=MYISAM ;


CREATE TABLE controllers (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	MANUFACTURER	VARCHAR(255) default NULL,
	NAME		VARCHAR(255) default NULL,
	CAPTION		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	VERSION		VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
		INDEX ID(ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;



CREATE TABLE devices (
	HARDWARE_ID	INTEGER NOT NULL,
	NAME		VARCHAR(50) NOT NULL,
	IVALUE		INTEGER default NULL,
	TVALUE		VARCHAR(255) default NULL,
	COMMENTS	TEXT,
		INDEX HARDWARE_ID (HARDWARE_ID),
		INDEX TVALUE (TVALUE)
) ENGINE=INNODB ;


CREATE TABLE drives (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	LETTER		VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
	FILESYSTEM	VARCHAR(255) default NULL,
	TOTAL		INTEGER default NULL,
	FREE		INTEGER default NULL,
	NUMFILES	INTEGER default NULL,
	VOLUMN		VARCHAR(255) default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;


CREATE TABLE files (
	NAME		VARCHAR(255) NOT NULL,
	VERSION		VARCHAR(255) NOT NULL,
	OS		VARCHAR(255) NOT NULL,
	CONTENT 	LONGBLOB NOT NULL,
		PRIMARY KEY  (NAME, OS, VERSION)
) ENGINE=MYISAM ;



CREATE TABLE inputs (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	`TYPE`		VARCHAR(255) default NULL,
	MANUFACTURER	VARCHAR(255) default NULL,
	CAPTION		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	INTERFACE	VARCHAR(255) default NULL,
	POINTTYPE	VARCHAR(255) default NULL,
		INDEX ID (ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;


CREATE TABLE memories (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	CAPTION		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	CAPACITY	VARCHAR(255) default NULL,
	PURPOSE		VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
	SPEED		VARCHAR(255) default NULL,
	NUMSLOTS	SMALLINT default NULL,
		INDEX ID(ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;


CREATE TABLE modems (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	NAME		VARCHAR(255) default NULL,
	MODEL		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
		INDEX ID(ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;


CREATE TABLE monitors (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	MANUFACTURER	VARCHAR(255) default NULL,
	CAPTION		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
	SERIAL		VARCHAR(255) default NULL,
		INDEX ID(ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;



CREATE TABLE networks (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
	TYPEMIB		VARCHAR(255) default NULL,
	SPEED		VARCHAR(255) default NULL,
	MACADDR		VARCHAR(255) default NULL,
	`STATUS`	VARCHAR(255) default NULL,
	IPADDRESS	VARCHAR(255) default NULL,
	IPMASK		VARCHAR(255) default NULL,
	IPGATEWAY	VARCHAR(255) default NULL,
	IPSUBNET	VARCHAR(255) default NULL,
	IPDHCP		VARCHAR(255) default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID),
		INDEX ID (ID),
		INDEX MACADDR (MACADDR),
		INDEX IPSUBNET (IPSUBNET)
) ENGINE=INNODB ;



CREATE TABLE network_devices(
	ID		INTEGER NOT NULL auto_increment,
	DESCRIPTION	VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
	MACADDR		VARCHAR(255) default NULL,
	`USER`		VARCHAR(255) default NULL,
		PRIMARY KEY (ID),
		INDEX MACADDR (MACADDR)
) ENGINE=MYISAM ;



CREATE TABLE operators (
	ID		VARCHAR(255) NOT NULL default '',
	FIRSTNAME	VARCHAR(255) default NULL,
	LASTNAME	VARCHAR(255) default NULL,
	PASSWD		VARCHAR(50) default NULL,
	ACCESSLVL	INTEGER default NULL,
	COMMENTS	text,
		PRIMARY KEY  (ID)
) ENGINE=MYISAM ;

CREATE TABLE ports (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	`TYPE`		VARCHAR(255) default NULL,
	NAME		VARCHAR(255) default NULL,
	CAPTION		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
		INDEX ID(ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;


CREATE TABLE printers (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	NAME		VARCHAR(255) default NULL,
	DRIVER		VARCHAR(255) default NULL,
	PORT		VARCHAR(255) default NULL,
		INDEX ID (ID),
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;


CREATE TABLE regconfig (
	ID		INTEGER NOT NULL auto_increment,
	NAME		VARCHAR(255) default NULL,
	REGTREE		INTEGER default NULL,
	REGKEY		text,
	REGVALUE	VARCHAR(255) default NULL,
		PRIMARY KEY  (ID),
		KEY NAME (NAME)
) ENGINE=MYISAM ;



CREATE TABLE registry (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	NAME		VARCHAR(255) default NULL,
	REGVALUE	VARCHAR(255) default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID),
		KEY NAME (NAME)
) ENGINE=INNODB ;



CREATE TABLE slots (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	NAME		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	DESIGNATION	VARCHAR(255) default NULL,
	PURPOSE		VARCHAR(255) default NULL,
	`STATUS`	VARCHAR(255) default NULL,
	PSHARE		tinyint(4) default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;



CREATE TABLE softwares (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	PUBLISHER	VARCHAR(255) default NULL,
	NAME		VARCHAR(255) default NULL,
	VERSION		VARCHAR(255) default NULL,
	FOLDER		text,
	COMMENTS	text,
	FILENAME	VARCHAR(255) default NULL,
	FILESIZE	INTEGER default '0',
	SOURCE		INTEGER default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID),
		INDEX NAME (NAME)
) ENGINE=INNODB ;




CREATE TABLE `sounds` (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	MANUFACTURER	VARCHAR(255) default NULL,
	NAME		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;



CREATE TABLE storages (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	MANUFACTURER	VARCHAR(255) default NULL,
	NAME		VARCHAR(255) default NULL,
	MODEL		VARCHAR(255) default NULL,
	DESCRIPTION	VARCHAR(255) default NULL,
	`TYPE`		VARCHAR(255) default NULL,
	DISKSIZE	INTEGER default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;



CREATE TABLE videos (
	ID		INTEGER NOT NULL auto_increment,
	HARDWARE_ID	INTEGER NOT NULL,
	NAME		VARCHAR(255) default NULL,
	CHIPSET		VARCHAR(255) default NULL,
	MEMORY		VARCHAR(255) default NULL,
	RESOLUTION	VARCHAR(255) default NULL,
		PRIMARY KEY  (ID, HARDWARE_ID)
) ENGINE=INNODB ;


CREATE TABLE devicetype (
	ID		INTEGER NOT NULL auto_increment,
	NAME		VARCHAR(255) default NULL,
		PRIMARY KEY  (ID)
) ENGINE=MYISAM ;


CREATE TABLE subnet (
	NETID		VARCHAR(15) NOT NULL,
	NAME		VARCHAR(255),
	ID		INTEGER,
	MASK		VARCHAR(255),
		PRIMARY KEY (NETID)
) ENGINE=MYISAM ;


CREATE TABLE locks(
	HARDWARE_ID	INTEGER NOT NULL PRIMARY KEY,
	ID		INTEGER DEFAULT NULL,
	SINCE		TIMESTAMP,
		INDEX SINCE (SINCE)
) ENGINE=HEAP ;
TRUNCATE TABLE locks;



CREATE TABLE dico_cat(
	NAME		VARCHAR(255) NOT NULL,
	PERMANENT	TINYINT DEFAULT 0,
		PRIMARY KEY(NAME)
) ENGINE=MYISAM ;


CREATE TABLE dico_ignored(
	EXTRACTED	VARCHAR(255) NOT NULL,
		PRIMARY KEY(EXTRACTED)
) ENGINE=MYISAM ;


CREATE TABLE dico_soft( 
	EXTRACTED	VARCHAR(255) NOT NULL,
	FORMATTED	VARCHAR(255) NOT NULL,
	PRIMARY KEY(EXTRACTED)
) ENGINE=MYISAM ;


CREATE TABLE deleted_equiv(
	DATE 		TIMESTAMP, 
	DELETED		VARCHAR(255) NOT NULL,
	EQUIVALENT	VARCHAR(255) default NULL
) ENGINE=MYISAM ;


CREATE TABLE download_available(
	FILEID		VARCHAR(255) NOT NULL PRIMARY KEY,
	NAME		VARCHAR(255) NOT NULL,
	PRIORITY	INTEGER NOT NULL,
	FRAGMENTS	INTEGER NOT NULL,
	SIZE		INTEGER NOT NULL,
	OSNAME		VARCHAR(255) NOT NULL,
	COMMENT 	TEXT
) ENGINE = INNODB;


CREATE TABLE download_enable(
	ID		INTEGER NOT NULL auto_increment PRIMARY KEY,
	FILEID		VARCHAR(255) NOT NULL,
	INFO_LOC	VARCHAR(255) NOT NULL,
	PACK_LOC	VARCHAR(255) NOT NULL,
	CERT_PATH	VARCHAR(255),
	CERT_FILE	VARCHAR(255)
) ENGINE = INNODB;


CREATE TABLE download_history(
	HARDWARE_ID	INTEGER NOT NULL,
	PKG_ID		INTEGER default NULL,
	PKG_NAME	VARCHAR(255),
		PRIMARY KEY(HARDWARE_ID, PKG_ID)
) ENGINE = INNODB;


CREATE TABLE conntrack(
	IP		VARCHAR(255),
	`TIMESTAMP`	TIMESTAMP,
		PRIMARY KEY(IP)
) ENGINE = HEAP;




DELETE FROM `config` WHERE name='GUI_VERSION';
DELETE FROM `config` WHERE name='IP_MIN_QUALITY';
INSERT INTO `config` VALUES ('FREQUENCY', 0, '', 'Specify the frequency (days) of inventories. (0: inventory at each login. -1: no inventory)');
INSERT INTO `config` VALUES ('PROLOG_FREQ', 24, '', 'Specify the frequency (hours) of prolog, on agents');
INSERT INTO `config` VALUES ('IPDISCOVER', 2, '', 'Max number of computers per gateway retrieving IP on the network');
INSERT INTO `config` VALUES ('INVENTORY_DIFF', 1, '', 'Activate/Deactivate inventory incremental writing');
INSERT INTO `config` VALUES ('IPDISCOVER_LATENCY', 100, '', 'Default latency between two arp requests');
INSERT INTO `config` VALUES ('INVENTORY_TRANSACTION', 1, '', 'Enable/disable db commit at each inventory section');
INSERT INTO `config` VALUES ('REGISTRY', 0, '', 'Activates or not the registry query function');
INSERT INTO `config` VALUES ('IPDISCOVER_MAX_ALIVE', 7, '','Max number of days before an Ip Discover computer is replaced');
INSERT INTO `config` VALUES ('DEPLOY', 1, '', 'Activates or not the automatic deployment option');
INSERT INTO `config` VALUES ('UPDATE', 0, '', 'Activates or not the update feature');
INSERT INTO `config` VALUES ('GUI_VERSION', 0, '4100', 'Version of the installed GUI and database');
INSERT INTO `config` VALUES ('TRACE_DELETED', 0, '', 'Trace deleted/duplicated computers (Activated by GLPI)');
INSERT INTO `config` VALUES ('LOGLEVEL', 0, '', 'ocs engine loglevel');
INSERT INTO `config` VALUES ('AUTO_DUPLICATE_LVL', 7, '', 'Duplicates bitmap');
INSERT INTO `config` VALUES ('DOWNLOAD', 0, '', 'Activate softwares auto deployment feature');
INSERT INTO `config` VALUES ('DOWNLOAD_CYCLE_LATENCY', 60, '', 'Time between two cycles (seconds)');
INSERT INTO `config` VALUES ('DOWNLOAD_PERIOD_LENGTH', 10, '', 'Number of cycles in a period');
INSERT INTO `config` VALUES ('DOWNLOAD_FRAG_LATENCY', 10, '', 'Time between two downloads (seconds)');
INSERT INTO `config` VALUES ('DOWNLOAD_PERIOD_LATENCY', 0, '', 'Time between two periods (seconds)');
INSERT INTO `config` VALUES ('DOWNLOAD_TIMEOUT', 30, '', 'Validity of a package (in days)');
INSERT INTO `config` VALUES ('LOCAL_SERVER', 0, 'localhost', 'Server address used for local import');
INSERT INTO `config` VALUES ('LOCAL_PORT', 80, '', 'Server port used for local import');

INSERT INTO `operators` VALUES ('admin','admin','admin','admin',1, 'Default administrator account');

GRANT ALL PRIVILEGES ON ocsweb.* TO ocs IDENTIFIED BY 'ocs';
GRANT ALL PRIVILEGES ON ocsweb.* TO ocs@localhost IDENTIFIED BY 'ocs';
